<?php

namespace Ebuy\Demo\Policies;

use Litepie\User\Contracts\UserPolicy;
use Ebuy\Demo\Models\Demo;

class DemoPolicy
{

    /**
     * Determine if the given user can view the demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function view(UserPolicy $user, Demo $demo)
    {
        if ($user->canDo('demo.demo.view') && $user->isAdmin()) {
            return true;
        }

        return $demo->user_id == user_id() && $demo->user_type == user_type();
    }

    /**
     * Determine if the given user can create a demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('demo.demo.create');
    }

    /**
     * Determine if the given user can update the given demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function update(UserPolicy $user, Demo $demo)
    {
        if ($user->canDo('demo.demo.edit') && $user->isAdmin()) {
            return true;
        }

        return $demo->user_id == user_id() && $demo->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Demo $demo)
    {
        return $demo->user_id == user_id() && $demo->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Demo $demo)
    {
        if ($user->canDo('demo.demo.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given demo.
     *
     * @param UserPolicy $user
     * @param Demo $demo
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Demo $demo)
    {
        if ($user->canDo('demo.demo.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
